(* Author:  Robbert van Renesse, Cornell University.
 *
 * Lexical analysis of .mli files.
 *)

{
open Mlipars

let lineno = ref 1
let incl_where = ref ""
let incl_buf = ref ""
}

rule token = parse
    ':'
      { COLON }
  | ';'
      { SEMICOLON }
  | ','
      { COMMA }
  | "'"
      { QUOTE }
  | '='
      { EQUALS } 
  | '*'
      { TIMES }
  | '|'
      { OR }
  | '.'
      { DOT }
  | '('
      { LB }
  | ')'
      { RB }
  | '{'
      { LCB }
  | '}'
      { RCB }
  | "->"
      { ARROW }
  | "of"
      { OF }
  | "mutable"
      { MUTABLE }
  | "type"
      { TYPE }
  | "val"
      { VAL }
  | "external"
      { EXTERNAL }
  | "exttype"
      { EXTTYPE }
  | "exception"
      { EXCEPTION }
  | ['a'-'z'] ['a'-'z' 'A'-'Z' '0'-'9' '_'] *
      { LC_IDENT(Lexing.lexeme lexbuf) }
  | ['A'-'Z'] ['a'-'z' 'A'-'Z' '0'-'9' '_'] *
      { UC_IDENT(Lexing.lexeme lexbuf) }
  | '"' [^ '"']* '"'
      { let s = Lexing.lexeme lexbuf in
	let len = String.length s in
	STRING(String.sub s 1 (len - 2))
      }
  | [' ' '\t' '\r']
      { token lexbuf }
  | "(*H_INCLUDE:"
      { incl_where := "h"; incl_buf := ""; includ lexbuf }
  | "(*C_INCLUDE:"
      { incl_where := "c"; incl_buf := ""; includ lexbuf }
  | "(*PRE_INCLUDE:"
      { incl_where := "pre"; incl_buf := ""; includ lexbuf }
  | "(*POST_INCLUDE:"
      { incl_where := "post"; incl_buf := ""; includ lexbuf }
  | "(*{"
      { token lexbuf }
  | "}*)"
      { token lexbuf }
  | "(*"
      { comment lexbuf }
  | '\n'
      { incr lineno; token lexbuf }
  | _ { Printf.fprintf stderr "Unexpected character in line %d: '%s'\n"
	!lineno (Lexing.lexeme lexbuf);
	raise (Failure "lexer: unexpected character") }
  | eof
      { EOF }

and includ = parse
  | "*)"
      { INCLUDE(!incl_where, !incl_buf) }
  | '*'
      { incl_buf := !incl_buf ^ "*"; includ lexbuf }
  | '\n'
      { incr lineno; incl_buf := !incl_buf ^ "\n"; includ lexbuf }
  | [^ '*' '\n']
      { incl_buf := !incl_buf ^ (Lexing.lexeme lexbuf); includ lexbuf }
  | eof
      { raise (Failure "lexer: EOF in INCLUDE comment") }

and comment = parse
    "(*"
      { comment lexbuf }
  | "*)"
      { token lexbuf }
  | '\n'
      { incr lineno; comment lexbuf }
  | _
      { comment lexbuf }
  | eof
      { raise (Failure "lexer: EOF in comment") }
